@@ -1,6 +1,6 @@
 // Filename: emulator.rs
-// Version:	 0.6
-// Date:	 09-07-2021 (DD-MM-YYYY)
+// Version:	 0.7
+// Date:	 14-11-2021 (DD-MM-YYYY)
 // Library:  gpcas_isa
 //
 // Copyright (c) 2021 Kai Rese
@@ -33,24 +33,19 @@ pub trait Emulator {
     /// Emulates the instruction at the given address. Returns `None` if program execution
     /// terminated.
     ///
-    /// If the requested address differs from the one of the next instruction in program order, the
-    /// instruction mustn't commit its result.
-    fn emulate_instruction(&mut self, address: usize) -> Option<Instruction>;
+    /// Non-committed instructions mustn't change the emulator state.
+    fn emulate_instruction(&mut self, address: usize, commit: bool) -> Option<Instruction>;
 
     /// Emulates the next instruction in program order. Returns `None` if program execution
     /// terminated.
     fn emulate_next_instruction(&mut self) -> Option<Instruction> {
-        self.emulate_instruction(self.get_current_ip())
+        self.emulate_instruction(self.get_current_ip(), true)
     }
 
     /// Returns the number of instructions the emulator has decoded.
     ///
     /// Every call to emulate an instruction is counted, even if the instruction doesn't retire.
     fn get_decoded_instruction_count(&self) -> usize;
-    /// Returns the number of instructions the emulator has executed and retired.
-    ///
-    /// Emulation of speculative instructions isn't counted for this.
-    fn get_retired_instruction_count(&self) -> usize;
     /// Returns the current instruction pointer address.
     fn get_current_ip(&self) -> usize;
     /// Returns the output the emulated program has written to stdout.
@@ -59,6 +54,47 @@ pub trait Emulator {
     fn get_start_address(&self) -> usize;
 }
 
+/// Additional functions used for interactive emulation.
+pub trait InteractiveEmulator: Emulator {
+    /// Read access to the entire memory of the emulated application.
+    fn get_memory(&self) -> &[u8];
+    /// Overwrite a block of memory with new content.
+    fn set_memory(&mut self, offset: usize, data: &[u8]);
+
+    /// Get the value of a general purpose register, extended to the native host's integer size.
+    fn get_register(&self, index: usize) -> usize;
+    /// Set a general purpose register to a new value. Might get truncated to the guest's integer
+    /// size.
+    fn set_register(&mut self, index: usize, value: usize);
+
+    /// Read access to a vector register.
+    ///
+    /// The length of the returned slice is dependent on the ISA. ISAs without vector support
+    /// should return a slice with zero length.
+    fn get_vector_register(&self, index: usize) -> &[u8];
+    /// Get a vector register to a new value.
+    ///
+    /// Slices shorter than the entire vector should cause the additional upper part of the vector
+    /// to be set to zero. ISAs with variable vector length support should set the register length
+    /// to the length of the value slice.
+    /// As an emulator should not call this for ISAs without vector support, the method is allowed
+    /// to panic in this case.
+    fn set_vector_register(&mut self, index: usize, value: &[u8]);
+
+    /// Emulate a directly input instruction.
+    ///
+    /// The default implementation works by overwriting the beginning of the emulated application's
+    /// memory, emulating from there, and writing the original data back. Because of this
+    /// behaviour, the instruction to be emulated should not be dependent on said memory. If you
+    /// want to lift this restriction, provide a custom implementation.
+    fn emulate_custom_input(&mut self, instruction: &[u8]) {
+        let memory_saved = Vec::from(&self.get_memory()[..instruction.len()]);
+        self.set_memory(0, instruction);
+        self.emulate_instruction(0, true);
+        self.set_memory(0, &memory_saved[..]);
+    }
+}
+
 /// Provides a continuous memory storage for multiple operands.
 pub struct OperandStorage {
     /// The actual memory storage.
